"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const summaryTemplate = '' +
    '=============================== Coverage summary ===============================\n' +
    'Total: {{ total.percent }}% covered\n' +
    'Statements: {{ statements.percent }}% ({{ statements.covered }}/{{ statements.total }})<% if(statements.skipped > 0) { print(", " + statements.skipped + " ignored") } %>\n' +
    'Branches: {{ branches.percent }}% ({{ branches.covered }}/{{ branches.total }})<% if(branches.skipped > 0) { print(", " + branches.skipped + " ignored") } %>\n' +
    'Functions: {{ functions.percent }}% ({{ functions.covered }}/{{ functions.total }})<% if(functions.skipped > 0) { print(", " + functions.skipped + " ignored") } %>\n' +
    'Lines: {{ lines.percent }}% ({{ lines.covered }}/{{ lines.total }})<% if(lines.skipped > 0) { print(", " + lines.skipped + " ignored") } %>\n' +
    '================================================================================\n';
class SimpleCoverageConfigBuilder {
    constructor(parent) {
        this.properties = {
            parent,
            configuration: this._defaultConfiguration()
        };
    }
    /**
     * The configuration group name
     */
    get _subConfigurationGroupName() {
        return 'coverage';
    }
    /**
     * Clear the configuration
     */
    clear() {
        this.properties.configuration = this._defaultConfiguration();
        return this;
    }
    /**
     * Enable the coverage
     */
    enable() {
        this.properties.configuration.enabled = true;
        return this;
    }
    /**
     * Disable the coverage
     */
    disable() {
        this.properties.configuration.enabled = false;
        return this;
    }
    /**
     * Set the enable value
     * @param [value] The new value. Set undefined to set put the default value
     */
    setEnabled(value) {
        this.properties.configuration.enabled = !!value;
        return this;
    }
    /**
     * Set the summary template.
     *
     * The template is a lodash template.
     * @param [value] The new value. Set undefined to set put the default value
     */
    summaryTemplate(value) {
        if (_.isNil(value)) {
            this.properties.configuration.summary.template = this._defaultConfiguration().summary.template;
        }
        else if (_.isString(value)) {
            this.properties.configuration.summary.template = value;
        }
        return this;
    }
    /**
     * Enable or not the summary
     * @param [value] The new value. Set undefined to set put the default value
     */
    enableSummary(value) {
        if (_.isNil(value)) {
            this.properties.configuration.summary.enabled = this._defaultConfiguration().summary.enabled;
        }
        else if (_.isBoolean(value)) {
            this.properties.configuration.summary.enabled = value;
        }
        else if (_.isString(value)) {
            this.properties.configuration.summary.enabled = value.toLowerCase() === 'true';
        }
        return this;
    }
    /**
     * Returns the parent builder
     */
    parentBuilder() {
        return this.properties.parent;
    }
    /**
     * Generate the object configuration
     */
    build() {
        const userConfig = {
            summary: {
                template: this.properties.configuration.summary.template
            }
        };
        if (!_.isNil(this.properties.configuration.enabled)) {
            userConfig.enabled = this.properties.configuration.enabled;
        }
        if (!_.isNil(this.properties.configuration.summary.enabled)) {
            userConfig.summary.enabled = this.properties.configuration.summary.enabled;
        }
        return _.merge({}, userConfig);
    }
    /**
     * Returns the default configuration
     */
    _defaultConfiguration() {
        return {
            enabled: undefined,
            summary: {
                enabled: undefined,
                template: summaryTemplate
            }
        };
    }
}
exports.SimpleCoverageConfigBuilder = SimpleCoverageConfigBuilder;
//# sourceMappingURL=coverage-config-builder.js.map