"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const coverage_config_builder_1 = require("./coverage-config-builder");
const project_config_builder_1 = require("./project-config-builder");
const reports_config_builder_1 = require("./reports-config-builder");
const test_config_builder_1 = require("./test-config-builder");
const _ = require("lodash");
/**
 * The configuration builder
 * In case you need to add a sub configuration builder, end you property name by ConfigBuilder, and it will
 * automatically added in the clear and build function.
 * To be handled by the build function, the builder need to have the property _subConfigurationGroupName defined
 */
class SimpleTestLauncherConfigBuilder {
    constructor() {
        this.properties = {
            configuration: this._defaultConfiguration()
        };
        this._createSubConfigurationBuilders();
    }
    /**
     * Returns the test configuration
     */
    testConfiguration() {
        return this.properties.testConfigBuilder;
    }
    /**
     * Returns the reports configuration
     */
    reportsConfiguration() {
        return this.properties.reportsConfigBuilder;
    }
    /**
     * Returns the coverage configuration
     */
    coverageConfiguration() {
        return this.properties.coverageConfigBuilder;
    }
    /**
     * Returns the project configuration
     */
    projectConfiguration() {
        return this.properties.projectConfigBuilder;
    }
    /**
     * Clear the configuration
     */
    clear() {
        this.properties.configuration = this._defaultConfiguration();
        // Use a special pattern
        _.each(_.filter(_.keysIn(this.properties), key => _.endsWith(key, 'ConfigBuilder')), key => {
            if (_.isObject(this.properties[key]) && _.isFunction(this.properties[key].clear)) {
                this.properties[key].clear();
            }
        });
        return this;
    }
    /**
     * Add a pre launch step function
     * @param value The step function to add
     */
    withPreLaunchStep(value) {
        if (_.isFunction(value)) {
            this.properties.configuration.steps.preLaunch.push({
                handler: value
            });
        }
        return this;
    }
    /**
     * Add a pre test step function
     * @param value The step function to add
     */
    withPreTestLaunchStep(value) {
        if (_.isFunction(value)) {
            this.properties.configuration.steps.preTestLaunch.push({
                handler: value
            });
        }
        return this;
    }
    /**
     * Add a post execution step function
     * @param value The step function to add
     */
    withPostExecutionStep(value) {
        if (_.isFunction(value)) {
            this.properties.configuration.steps.postExecution.push({
                handler: value
            });
        }
        return this;
    }
    /**
     * Add a clean up step function
     * @param value The step function to add
     */
    withCleanUpStep(value) {
        if (_.isFunction(value)) {
            this.properties.configuration.steps.cleanUp.push({
                handler: value
            });
        }
        return this;
    }
    /**
     * Generate the configuration
     */
    build() {
        const subConfiguration = {};
        // _subConfigurationGroupName
        _.each(_.filter(_.keysIn(this.properties), key => _.endsWith(key, 'ConfigBuilder')), key => {
            if (_.isObject(this.properties[key]) && _.isFunction(this.properties[key].build) && _.isString(this.properties[key]._subConfigurationGroupName)) {
                subConfiguration[this.properties[key]._subConfigurationGroupName] = this.properties[key].build();
            }
        });
        return _.assign({}, this.properties.configuration, subConfiguration);
    }
    _defaultConfiguration() {
        return {
            steps: {
                preLaunch: [],
                preTestLaunch: [],
                postExecution: [],
                cleanUp: []
            }
        };
    }
    /**
     * Create the configuration builders
     */
    _createSubConfigurationBuilders() {
        this.properties.coverageConfigBuilder = new coverage_config_builder_1.SimpleCoverageConfigBuilder(this);
        this.properties.testConfigBuilder = new test_config_builder_1.SimpleTestConfigBuilder(this);
        this.properties.reportsConfigBuilder = new reports_config_builder_1.SimpleReportsConfigBuilder(this);
        this.properties.projectConfigBuilder = new project_config_builder_1.SimpleProjectConfigBuilder(this);
    }
}
exports.SimpleTestLauncherConfigBuilder = SimpleTestLauncherConfigBuilder;
//# sourceMappingURL=test-launcher-config-builder.js.map