"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const logger_1 = require("./logger");
const _ = require("lodash");
const events_1 = require("events");
function lodashMergerWithIgnoreNil(objValue, srcValue) {
    if (_.isNil(srcValue)) {
        return objValue;
    }
}
/**
 * Jasmine runner
 */
class SimpleJasmineRunner extends events_1.EventEmitter {
    constructor() {
        super();
        this.logger = logger_1.createLogger('jasmine');
        this.properties = {
            instance: undefined,
            lastExecution: {
                success: false
            },
            specFileFilters: []
        };
    }
    get lastExecutionSucceed() {
        return this.properties.lastExecution.success;
    }
    /**
     * Initialize the runner
     * @param configuration The runner configuration
     */
    async initialize(configuration) {
        this.properties.projectDirectoryPath = configuration.projectDirectoryPath;
        this.properties.specFileFilters = configuration.specFileFilters;
        this.properties.specDirPath = configuration.specDirPath;
        this.properties.features = configuration.features;
        this.properties.reportDirectoryPath = configuration.reportDirectoryPath;
        this.logger.debug(`Create and initialize the jasmine instance`);
        await this.createAndInitInstance();
    }
    /**
     * Start the jasmine execution
     */
    async start(options) {
        // @ts-ignore
        this.properties.instance.onComplete(passed => {
            this.properties.lastExecution.success = !!passed;
            setImmediate(() => this.onJasmineFinished());
        });
        // @ts-ignore
        this.properties.instance.env.executionContext = options.executionContext;
        this.logger.info(`Start the execution`);
        // @ts-ignore
        this.properties.instance.execute();
    }
    run(options) {
        return new Promise((resolve, reject) => {
            this.once('finished', resolve);
            this.start(options)
                .catch(reject);
        });
    }
    /**
     * Generate the jasmine configuration
     */
    async generateConfiguration() {
        const defaultJasmineConfiguration = {
            spec_dir: this.properties.specDirPath,
            spec_files: [
                "**/*[sS]pec.js",
                '*[sS]pec.js',
                "**/*[sS]pec.ts",
                '*[sS]pec.ts'
            ],
            stopSpecOnExpectationFailure: false,
            random: false
        };
        const jasmineConfiguration = _.mergeWith({}, defaultJasmineConfiguration, {
            features: this.properties.features
        }, lodashMergerWithIgnoreNil);
        return {
            configuration: jasmineConfiguration
        };
    }
    /**
     * Filters the spec files with user custom filters
     * @param options Options
     */
    async filterTestSpecFiles(options) {
        if (!this.properties.specFileFilters || this.properties.specFileFilters.length === 0) {
            return {
                filePaths: options.filePaths
            };
        }
        // Generate the filters
        const filters = this.properties.specFileFilters.map(filter => {
            if (_.isString(filter) || _.isRegExp(filter)) {
                let regexpFilter = filter;
                if (_.isString(filter)) {
                    // Create a regexp
                    regexpFilter = new RegExp(filter);
                }
                return async (filePath) => {
                    const matches = filePath.match(regexpFilter);
                    const ignoreFile = _.isArray(matches) && matches.length > 0;
                    return {
                        ignore: ignoreFile
                    };
                };
            }
            else {
                return filter;
            }
        });
        const filePaths = [];
        for (const filePath of options.filePaths) {
            let keepFile = true;
            for (const filter of filters) {
                const result = await filter(filePath);
                if (result && result.ignore === true) {
                    keepFile = false;
                }
            }
            if (keepFile) {
                filePaths.push(filePath);
            }
        }
        return {
            filePaths
        };
    }
    /**
     * Create the jasmine reports
     */
    async createReporters() {
        const reporters = [];
        if (_.get(this.properties, 'features.reports.enableJunitXml')) {
            const jasmineReporters = require('jasmine-reporters'), reporter = new jasmineReporters.JUnitXmlReporter({
                savePath: `${this.properties.reportDirectoryPath}/junit/xml`,
                consolidateAll: false
            });
            reporters.push(reporter);
        }
        if (_.get(this.properties, 'features.reports.enableHtml')) {
            const Jasmine2HtmlReporter = require('protractor-jasmine2-html-reporter'), reporter = new Jasmine2HtmlReporter({
                savePath: `${this.properties.reportDirectoryPath}/html`,
                screenshotsFolder: 'images',
                takeScreenshots: false
            });
            reporters.push(reporter);
        }
        return {
            reporters
        };
    }
    /**
     * Create and initialize the jasmine instance
     */
    async createAndInitInstance() {
        const Jasmine = require('jasmine');
        const jasmineOptions = {
            projectBaseDir: this.properties.projectDirectoryPath
        };
        const jasmineInstance = new Jasmine(jasmineOptions);
        this.logger.debug(`Generate configuration`);
        const { configuration } = await this.generateConfiguration();
        jasmineInstance.loadConfig(configuration);
        this.logger.debug(`Generate reporters`);
        const { reporters } = await this.createReporters();
        reporters.forEach(reporter => jasmineInstance.addReporter(reporter));
        this.logger.debug(`Filter spec files`);
        const { filePaths } = await this.filterTestSpecFiles({
            filePaths: jasmineInstance.specFiles
        });
        this.logger.debug(`Use those files:\n${filePaths.join('\n')}\n`);
        jasmineInstance.specFiles = filePaths;
        this.properties.instance = jasmineInstance;
    }
    /**
     * Executed when jasmin finished
     */
    onJasmineFinished() {
        this.emit('finished', {
            _origin: this,
            success: this.properties.lastExecution.success
        });
    }
}
exports.SimpleJasmineRunner = SimpleJasmineRunner;
//# sourceMappingURL=simple-jasmine-runner.js.map